setwd ("XXXXXXXX")
source("R/global_function.R")
library("pdftools")

options(scipen = 10000)

##### Connect to the PostgreSQL DB and extract all project info ####
con <- dbConnect(PostgreSQL(), host="host", dbname = "dbname", port = port, user = "user", password = "psw")
q_proj = "select pro.pkid as id_project, 
prog.name as programme_name, prog1.name as programme1_name, prog2.name as programme2_name,
fund.name as funding_source, funding, cost,
cat.name as category, key.name as keyword,
pro.acronym as code, pro.title as title, summary, pro.website, pro.active as status,
inst.name as coord_name, inst.acronym as coord_acronym, 
country.name as coord_country, continent.name as coord_continent,
inst2.name as inv_name, inst2.acronym as inv_acronym, 
country2.name as inv_country, continent2.name as inv_continent,
pro.start_year, pro.end_year,
area.id as id_area, area.name as area, suba.id as id_subarea, suba.name as subarea, div.name as division, div.id as id_division
from project as pro
left join rel_division_project as rel_div_pro on rel_div_pro.fkproject = pro.pkid
left join division as div on div.id = rel_div_pro.fkdivision
left join subarea as suba on suba.id = div.fksubarea
left join area on area.id = suba.fkarea
left join institution as inst on inst.id = pro.fkcoord_institution
left join country on country.pkid = inst.fkcountry
left join continent on continent.id = country.fkcontinent
left join rel_institution_project as rel_inst_pro on rel_inst_pro.fkproject = pro.pkid
left join institution as inst2 on inst2.id = rel_inst_pro.fkinstitution
left join country as country2 on country2.pkid = inst2.fkcountry
left join continent as continent2 on continent2.id = country2.fkcontinent
left join funding_source as fund on fund.id = pro.fkfunding_source
left join programme as prog on prog.id = pro.fkprogramme
left join programme1 as prog1 on prog1.id = pro.fkprogramme1
left join programme2 as prog2 on prog2.id = pro.fkprogramme2
left join rel_category_project as rel_cat_pro on rel_cat_pro.fkproject = pro.pkid
left join category as cat on cat.pkid = rel_cat_pro.fkcategory
left join rel_keyword_project as rel_key_pro on rel_key_pro.fkproject = pro.pkid
left join keyword as key on key.id = rel_key_pro.fkkeyword"

proj_dat = st_read(con, query = q_proj) %>%
  mutate(category = ifelse(category == "Sea Food Processing", "Seafood Processing", category)) %>%
  mutate(title_origin = title,
         code = formatting_string(ifelse(code == "", NA, as.character(code))),
         title = formatting_string(ifelse(title == "", NA, title)),
         summary = ifelse(summary == "", NA, summary),
         website = ifelse(website == "", NA, website),
         funding_source = ifelse(funding_source == "", NA, as.character(funding_source)),
         coord_country = ifelse(coord_country == "", NA, as.character(coord_country)),
         start_year = ifelse(start_year == "" | is.na(start_year), NA, as.numeric(as.character(start_year))),
         end_year = ifelse(end_year == "" | is.na(end_year), NA, as.numeric(as.character(end_year)))) 
lapply(dbListConnections(PostgreSQL()), dbDisconnect)

# Assign double category and code ###
## category
proj_category = proj_dat %>%
  distinct(id_project, category) %>%
  mutate(n = 1) %>%
  reshape2:::dcast(id_project ~ category, value.var = "n", fun.aggregate = sum)
proj_category$category_cl = apply(proj_category[,2:ncol(proj_category)], 1, function(x){
  xref = which(x != 0) 
  paste(colnames(proj_category)[2:ncol(proj_category)][xref], collapse = " + ")
})
proj_category = proj_category %>%
  distinct(id_project, category_cl)

## code
cat_code = read.xlsx("data/category_code.xlsx")
proj_cat_code = proj_dat %>%
  distinct(id_project, category) %>%
  mutate(n = 1) %>%
  left_join(cat_code, by = "category") %>%
  distinct(id_project, category_code, n) %>%
  reshape2:::dcast(id_project ~ category_code, value.var = "n", fun.aggregate = sum)
proj_cat_code$category_code_cl = apply(proj_cat_code[,2:ncol(proj_cat_code)], 1, function(x){
  xref = which(x != 0) 
  paste(colnames(proj_cat_code)[2:ncol(proj_cat_code)][xref], collapse = "+")
}) 
proj_cat_code = proj_cat_code %>% 
  distinct(id_project, category_code_cl)

proj_category = proj_category %>%
  inner_join(proj_cat_code, by = "id_project")

proj_dat = proj_dat%>%
  inner_join(proj_category %>%
  distinct(id_project, category_cl, category_code_cl), by = "id_project")


# Database overview  ####

## summary data
total_project = length(unique(proj_dat$id_project))
period = range(c(proj_dat$start_year, proj_dat$end_year), na.rm = T)

## time series by categories
proj_dat_cat = proj_dat %>% 
  # mutate(start_year2 = ifelse(is.na(start_year), "Not Available", start_year)) %>% #aggiunto NT
  mutate(start_year2 = ifelse(start_year < 2000, "<2000", start_year)) %>%
  filter(!is.na(start_year2)) %>%
  distinct(id_project, category_code_cl, start_year2) %>%
  group_by(start_year2, category_code_cl) %>%
  dplyr:::summarise(nproj = length(unique(id_project))) %>%
  mutate(category_code_cl = factor(category_code_cl, level = rev(sort(unique(proj_dat$category_code_cl)))))

proj_dat_cat_c = proj_dat_cat %>%
  # dcast(category_cl + category_code_cl~start_year2, value.var = "nproj", sum)
  dcast(category_code_cl~start_year2, value.var = "nproj", sum)
write.xlsx(proj_dat_cat_c, "results/tables/category_year.xlsx")

p_cat = ggplot() +
  geom_tile(data = proj_dat_cat, aes(start_year2, category_code_cl, fill = nproj)) + 
  coord_fixed() +
  scale_fill_viridis(breaks = round(seq(1, max(proj_dat_cat$nproj, na.rm = T), length.out = 5))) +
  xlab("") + ylab("") +
  theme_bw() +
  theme(axis.text.x = element_text(angle = 90, size = 10),
        legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm")) + 
  guides(fill = guide_colourbar(title = "N° of projects", size = 7))
ggsave("results/plots/category_year.pdf", p_cat, width = 25, height = 20, units = "cm", dpi = 300, device = cairo_pdf) #####Fig.4 - Num. of projects by category and starting year 
knitr::plot_crop("results/plots/category_year.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/category_year.pdf", dpi = 300)
png::writePNG(bitmap, "results/plots/category_year.png", dpi=300)
rm(bitmap); rm(p_cat); gc() 


# coordinator/involved country and funding source ####

proj_source = proj_dat %>%
  distinct(id_project, funding_source) %>%
  mutate(funding_source = ifelse(is.na(funding_source), "Not Available", funding_source)) %>%
  group_by(funding_source) %>%
  dplyr:::summarise(nproj = length(id_project))

proj_source_dt = proj_source %>%
  arrange(-nproj) %>%
  mutate(prop = nproj/sum(nproj)*100) %>%
  mutate(lab.ypos = cumsum(prop) - 0.5*prop,
         st_prop = st_fun(prop),
         nchar = nchar(as.character(nproj)),
         st_nchar = st_fun(nchar(as.character(nproj)))) %>%
  arrange(nproj) %>%
  mutate(funding_source = factor(funding_source, levels = funding_source))

p_source = ggplot() +
  geom_bar(data = proj_source_dt, aes(x = "", y = prop, fill = funding_source), width = 1, stat = "identity", color = "white", size = 0.3, show.legend = F) +
  geom_label_repel(data = proj_source_dt, aes(x = "", y = lab.ypos, label = paste0(funding_source, 
                                                                                   " \n Projects: ", nproj,
                                                                                   " \n", round(prop, 2), " %")),
                   color = "black", direction = "both", nudge_y = -0.5, nudge_x = 0.5, size = 7) +
  coord_polar("y", start = 0) +
  scale_fill_viridis(discrete = T) +
  theme_void() + guides(fill = guide_colorbar(title = "", override.aes = list(colour = 1, size = 0.5)))

ggsave("results/plots/source_pie.pdf", p_source, device = cairo_pdf, dpi = 300, width = 10, height = 10)##### Fig.6 left - Num. of projects by funding source 
knitr::plot_crop("results/plots/source_pie.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/source_pie.pdf", dpi = 300)
png::writePNG(bitmap, "results/plots/source_pie.png", dpi=300)
rm(bitmap); rm(p_source); gc()


# Budget ####
## budget category composition

proj_dat_budget = proj_dat %>% 
  mutate(budget_category = ifelse(funding < 100000, "<100K", 
                                  ifelse(funding >= 100000 & funding < 500000, "100-500K", ">500K")))
proj_bud = proj_dat_budget %>%
  filter(!is.na(budget_category)) %>%
  distinct(id_project, category_cl, budget_category, funding) %>%
  group_by(category_cl, budget_category) %>%
  dplyr:::summarise(nproj = length(unique(id_project)),
                    funding_tot = sum(funding, na.rm = T)/1000000)
proj_bud_c_nproj = proj_bud %>%
  dcast(category_cl ~ budget_category, value.var = "nproj", fun.aggregate = sum)
proj_bud_c_nproj = proj_bud %>%
  dcast(category_cl ~ budget_category, value.var = "nproj", fun.aggregate = sum)
proj_bud_c_nproj[,2:ncol(proj_bud_c_nproj)] = round(proj_bud_c_nproj[,2:ncol(proj_bud_c_nproj)]/apply(proj_bud_c_nproj[,2:ncol(proj_bud_c_nproj)], 1, sum),2)
proj_bud_c_fun = proj_bud %>%
  dcast(category_cl ~ budget_category, value.var = "funding_tot", fun.aggregate = sum)
proj_bud_c_fun[,2:ncol(proj_bud_c_fun)] = round(proj_bud_c_fun[,2:ncol(proj_bud_c_fun)]/apply(proj_bud_c_fun[,2:ncol(proj_bud_c_fun)], 1, sum),2)
proj_bud_c = proj_bud_c_nproj %>%
  inner_join(proj_bud_c_fun, by = c("category_cl"))
write.xlsx(proj_bud_c, "results/tables/project_budget_proportion.xlsx")

proj_bud_category = proj_dat_budget %>%
  distinct(id_project, budget_category) %>%
  mutate(budget_category = ifelse(is.na(budget_category), "Not Available", budget_category)) %>%
  group_by(budget_category) %>%
  dplyr:::summarise(nproj = length(id_project))

proj_bud_category_dt = proj_bud_category %>%
  arrange(-nproj) %>%
  mutate(prop = nproj/sum(nproj)*100) %>%
  mutate(lab.ypos = cumsum(prop) - 0.5*prop,
         st_prop = st_fun(prop),
         nchar = nchar(as.character(nproj)),
         st_nchar = st_fun(nchar(as.character(nproj)))) %>%
  arrange(nproj) %>%
  mutate(budget_category = factor(budget_category, levels = budget_category))

p_source = ggplot() +
  geom_bar(data = proj_bud_category_dt, aes(x = "", y = prop, fill = budget_category), width = 1, stat = "identity", color = "white", size = 0.3, show.legend = F) +
  geom_label_repel(data = proj_bud_category_dt, aes(x = "", y = lab.ypos, label = paste0(budget_category, 
                                                                                         " \n Projects: ", nproj,
                                                                                         " \n", round(prop, 2), " %")),
                   color = "black", direction = "both", nudge_y = -0.5, nudge_x = 0.5) +
  coord_polar("y", start = 0) +
  scale_fill_viridis(discrete = T) +
  theme_void() + guides(fill = guide_colorbar(title = "", override.aes = list(colour = 1, size = 0.5)))

ggsave("results/plots/budget_category_pie.pdf", p_source, device = cairo_pdf, dpi = 300)##### Fig.6 right - Number of projects by funding category 
knitr::plot_crop("results/plots/budget_category_pie.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/budget_category_pie.pdf", dpi = 300)
png::writePNG(bitmap, "results/plots/budget_category_pie.png", dpi=300)
rm(bitmap); rm(p_source); gc()

############################################ MAPS ############################################################################################
# load spatial layers ####
world <- ne_countries(scale = "medium", returnclass = "sf")
areas <- st_read("data/FAO_AREAS.shp")
major = areas %>% filter(F_LEVEL == "MAJOR")
subarea = areas %>% filter(F_LEVEL == "SUBAREA")
division = read_sf("data/div3857s.shp")
division = division %>% st_transform(wgs)
subdivison = areas %>% filter(F_LEVEL == "SUBDIVISION")
subunit = areas %>% filter(F_LEVEL == "SUBUNIT")

eu_ms = read.xlsx("data/eu_member_state.xlsx")

major_ref = major %>%
  st_set_geometry(NULL) %>%
  distinct(F_AREA, NAME_EN) %>%
  filter(F_AREA %in% c(27, 34, 37))

major_ref$NAME_NEW = c(paste(toupper("Mediterranean and Black Sea"), "(FAO Area 37)"),
                       paste(toupper("Atlantic Northeast"), "(FAO Area 27)"),
                       paste(toupper("Atlantic Eastern Central"), "(FAO Area 34)"))

xmajor = major %>% 
  filter(F_AREA %in% major_ref$F_AREA) %>%
  inner_join(major_ref, by = "F_AREA")

pmajor = ggplot() + 
  geom_sf(data = xmajor, aes(fill = NAME_NEW)) + 
  geom_sf(data = world, size = 0.1) +
  coord_sf(xlim = c(-25, 50), ylim = c(20, 80)) + 
  scale_fill_viridis(discrete = T) +
  theme_bw() +
  xlab("") +
  theme(legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm")) + 
  guides(fill = guide_legend(title = "", nrow = 1)) + 
  ggtitle(label = "", subtitle = "Major Fishing Area")

xsubarea = subarea %>%
  inner_join(major_ref, by = "F_AREA")

psubarea = ggplot() + 
  geom_sf(data = xsubarea, aes(fill = NAME_NEW)) + 
  geom_sf(data = world, size = 0.1) +
  coord_sf(xlim = c(-25, 50), ylim = c(20, 80)) + 
  scale_fill_viridis(discrete = T) +
  theme_bw() +
  xlab("") +
  theme(legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm")) + 
  guides(fill = guide_legend(title = "", nrow = 1)) + 
  ggtitle(label = "", subtitle = "Subarea")

parea = ggplot() + 
  geom_sf(data = division, aes(fill = AREA)) + 
  geom_sf(data = world, size = 0.1) +
  coord_sf(xlim = c(-25, 50), ylim = c(20, 80)) + 
  scale_fill_viridis(discrete = T) +
  theme_bw() +
  xlab("") +
  theme(legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm")) + 
  guides(fill = guide_legend(title = "", nrow = 1)) + 
  ggtitle(label = "", subtitle = "Division")

pcombo = ggarrange(plotlist= list(pmajor, psubarea, parea), nrow = 1, ncol = 3, common.legend = T, legend = "bottom", align = "hv")

ggsave("results/plots/marineareas.pdf", pcombo, device = cairo_pdf, width = 10) #Fig.2 - Major marine areas, subareas and divisions
knitr::plot_crop("results/plots/marineareas.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/marineareas.pdf", dpi = 300)
png::writePNG(bitmap, "results/plots/marineareas.png", dpi = 300)
rm(bitmap); rm(pcombo); gc()

## country map
country_coord_raw = proj_dat %>%
  distinct(id_project, coord_country) 

country_inv = proj_dat %>%
  filter(!is.na(inv_country) & inv_country != "Not available") %>%
  distinct(id_project, inv_country) %>%
  anti_join(country_coord_raw, by = c("id_project", "inv_country" = "coord_country"))

country_coord = country_coord_raw %>%
  group_by(coord_country) %>%
  dplyr:::summarise(nproj = length(id_project)) %>%
  ungroup() %>%
  dplyr:::rename("country" = "coord_country") %>%
  mutate(variable = "Coordinator")

country_inv = country_inv %>%
  group_by(inv_country) %>%
  dplyr:::summarise(nproj = length(id_project)) %>%
  ungroup() %>%
  dplyr:::rename("country" = "inv_country") %>%
  mutate(variable = "Partner")
### Europe
coord_inv = bind_rows(country_coord, country_inv) %>%
  mutate(nproj_cl = as.character(cut(nproj, format(unique(classIntervals(nproj, n = 7, style = "jenks", )$brks), scientific = F), 
                                     right = T, include.lowest = T, ordered_result = T, dig.lab = 10))) %>%
  arrange(nproj) %>%
  mutate(nproj_cl = factor(nproj_cl, levels = unique(nproj_cl))) %>%
  mutate(country = ifelse(country == "Russian Federation", "Russia", country))

coords = world %>% 
  inner_join(coord_inv %>% 
               filter(variable == "Coordinator"), by = c("admin" = "country"))
involvs = world %>% 
  inner_join(coord_inv %>% 
               filter(variable == "Partner"), by = c("admin" = "country")) 
coord_invol_map = bind_rows(coords, involvs)

bb = st_bbox(coord_invol_map)
xrange = bb[c(1, 3)]
yrange = bb[c(2, 4)]

p_coord_inv = ggplot() + 
  geom_sf(data = world, size = 0.1) +
  geom_sf(data = coord_invol_map, aes(fill = nproj_cl), colour = "white", size = 0.1) +
  coord_sf(xlim = c(-25, 50), ylim = c(20, 80)) + 
  scale_fill_viridis(discrete = T) +
  theme_bw() +
  theme(legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm")) + 
  guides(fill = guide_legend(title = "N° of projects", nrow = 2)) +
  facet_wrap(~variable)


ggsave("results/plots/coordinator_involved_europe.pdf", p_coord_inv, width = 20, height = 20, units = "cm", dpi = 300, device = cairo_pdf) ###Fig.5 - Num.of project by coordinator country and involved country. Bounding box centered on Europe
knitr::plot_crop("results/plots/coordinator_involved_europe.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/coordinator_involved_europe.pdf", dpi = 300)
png::writePNG(bitmap, "results/plots/coordinator_involved_europe.png", dpi = 300)
rm(bitmap); rm(p_coord_inv); gc()


## budget category and funding type map
proj_bud_source_coord = proj_dat_budget %>%
  filter(!is.na(budget_category)) %>%
  distinct(id_project, category_cl, coord_country, budget_category, funding_source, funding) %>%
  group_by(coord_country, funding_source, budget_category) %>%
  dplyr:::summarise(nproj = length(unique(id_project))) 
ints = classIntervals(proj_bud_source_coord$nproj, n = 7, style = "jenks", )$brks
ints[which.min(ints)] = 1
proj_bud_source_coord = proj_bud_source_coord %>%
  mutate(nproj_cl = cut(nproj, ints, include.lowest = T))

xx_map = world %>%
  inner_join(proj_bud_source_coord %>%
               mutate(coord_country = ifelse(coord_country == "Russian Federation", "Russia", coord_country)), by = c("admin" = "coord_country"))

ptot = ggplot() + 
  geom_sf(data = world, size = 0.1) +
  geom_sf(data = xx_map %>%
            mutate(budget_category = factor(budget_category, level = c("<100K", "100-500K", ">500K"))), aes(fill = nproj_cl), colour = "white", size = 0.1) +
  coord_sf(xlim = c(-25, 45), ylim = c(20, 80)) + 
  scale_fill_viridis(discrete = T) +
  theme_bw() +
  theme(axis.text.x = element_text(size = 5),
        legend.position = "bottom",
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        plot.margin=unit(c(1,1,1,1), "cm"),
        plot.title = element_text(size = 16)) + 
  guides(fill = guide_legend(title = "N° of projects", nrow = 1)) +
  facet_grid(budget_category~funding_source) +
  ggtitle("")

ggsave("results/plots/budget_category_funding_source_country.pdf", ptot, width = 50, height = 20, units = "cm", dpi = 600, device = cairo_pdf) ###Fig.7 - Num.of projects by coordinator country, funding category and funding source
knitr::plot_crop("results/plots/budget_category_funding_source_country.pdf", quiet = TRUE)
bitmap <- pdftools::pdf_render_page("results/plots/budget_category_funding_source_country.pdf", dpi = 600)
png::writePNG(bitmap, "results/plots/budget_category_funding_source_country.png", dpi = 300)
rm(bitmap); rm(ptot); gc()
