# libraries and functions####
library(RMySQL)  
library(stringr)
library(RPostgreSQL)
library(rpostgis)
library(dplyr)
library(sf)

formatting_string <- function(x){
  xx = iconv(x, "UTF-8", 'ASCII//TRANSLIT',sub='x')
  xx = toupper(xx)
  xx = gsub('[^[:alnum:]]',"", xx)
  return(xx)
}

setwd("XXXXXX")

# extract all project info ####
con <- dbConnect(PostgreSQL(), host="XXXX", dbname = "dbname", port = port, user = "user", password = "password")
q_proj = "select pro.pkid as id_project, 
prog.name as programme_name, prog1.name as programme1_name, prog2.name as programme2_name,
fund.name as funding_source, funding, cost,
cat.name as category, key.name as keyword,
pro.acronym as code, pro.title as title, summary, pro.website, pro.active as status,
inst.name as coord_name, inst.acronym as coord_acronym, 
country.name as coord_country, continent.name as coord_continent,
inst2.name as inv_name, inst2.acronym as inv_acronym, 
country2.name as inv_country, continent2.name as inv_continent,
pro.start_year, pro.end_year,
area.id as id_area, area.name as area, suba.id as id_subarea, suba.name as subarea, div.name as division, div.id as id_division
from project as pro
left join rel_division_project as rel_div_pro on rel_div_pro.fkproject = pro.pkid
left join division as div on div.id = rel_div_pro.fkdivision
left join subarea as suba on suba.id = div.fksubarea
left join area on area.id = suba.fkarea
left join institution as inst on inst.id = pro.fkcoord_institution
left join country on country.pkid = inst.fkcountry
left join continent on continent.id = country.fkcontinent
left join rel_institution_project as rel_inst_pro on rel_inst_pro.fkproject = pro.pkid
left join institution as inst2 on inst2.id = rel_inst_pro.fkinstitution
left join country as country2 on country2.pkid = inst2.fkcountry
left join continent as continent2 on continent2.id = country2.fkcontinent
left join funding_source as fund on fund.id = pro.fkfunding_source
left join programme as prog on prog.id = pro.fkprogramme
left join programme1 as prog1 on prog1.id = pro.fkprogramme1
left join programme2 as prog2 on prog2.id = pro.fkprogramme2
left join rel_category_project as rel_cat_pro on rel_cat_pro.fkproject = pro.pkid
left join category as cat on cat.pkid = rel_cat_pro.fkcategory
left join rel_keyword_project as rel_key_pro on rel_key_pro.fkproject = pro.pkid
left join keyword as key on key.id = rel_key_pro.fkkeyword"


proj_dat = st_read(con, query = q_proj) %>%
  mutate(category = ifelse(category == "Sea Food Processing", "Seafood Processing", category)) %>%
  mutate(title_origin = title,
         code = formatting_string(ifelse(code == "", NA, as.character(code))),
         title = formatting_string(ifelse(title == "", NA, title)),
         summary = ifelse(summary == "", NA, summary),
         website = ifelse(website == "", NA, website),
         funding_source = ifelse(funding_source == "", NA, as.character(funding_source)),
         coord_country = ifelse(coord_country == "", NA, as.character(coord_country)),
         start_year = ifelse(start_year == "" | is.na(start_year), NA, as.numeric(as.character(start_year))),
         end_year = ifelse(end_year == "" | is.na(end_year), NA, as.numeric(as.character(end_year)))) 
lapply(dbListConnections(PostgreSQL()), dbDisconnect)


data <-proj_dat %>% 
  mutate(coord_name_acronym=ifelse(is.na(coord_name),NA,paste0(coord_name,"(", coord_acronym,")")))%>% 
  mutate(inv_name_acronym=ifelse(is.na(inv_name),NA,paste0(inv_name,"(", inv_acronym,")")))

data<-data[order(data$category), ]

data <-data %>% 
  group_by(id_project) %>% 
  mutate(category_by_proj = paste0(unique(category), collapse = "/")) %>% 
  mutate(keyword_by_proj = paste0(unique(keyword), collapse = "/"))%>% 
  mutate(area_by_proj = paste0(unique(area), collapse = "/"))%>% 
  mutate(subarea_by_proj = paste0(unique(subarea), collapse = "/"))%>% 
  mutate(division_by_proj = paste0(unique(division), collapse = "/"))%>% 
  mutate(inv_name_acronym_by_proj = ifelse(is.na(inv_name),NA,paste0(unique(inv_name_acronym), collapse = "/")))%>% 
  mutate(inv_country_by_proj = ifelse(is.na(inv_name),NA,paste0(unique(inv_country), collapse = "/")))%>% 
  mutate(inv_continent_by_proj = ifelse(is.na(inv_name),NA,paste0(unique(inv_continent), collapse = "/")))
  # mutate(inv_country_by_proj_2 = ifelse(is.na(inv_name),NA,paste0(inv_country, collapse = "/")))%>% #to repeat the country for each involved institution
  # mutate(inv_continent_by_proj_2 = ifelse(is.na(inv_name),NA,paste0(inv_continent, collapse = "/")))#to repeat the continent for each involved institution

data<-data[,c("id_project","code","category_by_proj","title_origin","website","summary","funding_source","funding",
              "start_year","end_year","programme_name","programme1_name","programme2_name","keyword_by_proj","coord_name_acronym","coord_country","coord_continent",
              "inv_name_acronym_by_proj","inv_country_by_proj","inv_continent_by_proj","area_by_proj","subarea_by_proj","division_by_proj")]   

sci_data<-data %>% distinct(id_project, .keep_all= TRUE)
names(sci_data) <- c('project ID', 'acronym', 'category', 'title', 'website', 'summary', 'funding_source', 'funding', 
                     'start_year', 'end_year', 'programme','programme1','programme2','keyword', 'coord_institution', 'coord_country','coord_continent',
                     'inv_institution','inv_country','inv_continent','area','subarea','division')

write.csv2(sci_data, "Bluebio_db_28082022.csv", na ="",row.names = FALSE )